@@ -23,6 +23,13 @@ enum cg_version_t {
 	CGROUP_DISK = 0xFF,
 };
 
+enum cg_setup_mode_t {
+	CGROUP_MODE_UNK = 0,
+	CGROUP_MODE_LEGACY,
+	CGROUP_MODE_HYBRID,
+	CGROUP_MODE_UNIFIED,
+};
+
 /**
  * Flags for cgroup_delete_cgroup_ext().
  */
@@ -636,6 +643,56 @@ int cgroup_list_mount_points(const enum cg_version_t cgrp_version,
 int cgroup_get_controller_version(const char * const controller,
 				  enum cg_version_t * const version);
 
+/**
+ * Get the current group setup mode (legacy/unified/hybrid)
+ *
+ * @return CGROUP_MODE_UNK on failure and setup mode on success
+ */
+enum cg_setup_mode_t cgroup_setup_mode(void);
+
+/**
+ * Return the number of controllers for the specified cgroup in libcgroup
+ * internal structures.
+ *
+ * @param cgroup
+ * @return Count of the controllers or -1 on error.
+ */
+int cgroup_get_controller_count(struct cgroup *cgroup);
+
+/**
+ * Return requested controller from given group
+ *
+ * @param cgroup
+ * @param index The index into the cgroup controller list
+ */
+struct cgroup_controller *cgroup_get_controller_by_index(struct cgroup *cgroup, int index);
+
+/**
+ * Given a controller pointer, get the name of the controller
+ *
+ * @param controller
+ * @return controller name string, NULL if there's an error
+ */
+char *cgroup_get_controller_name(struct cgroup_controller *controller);
+
+/**
+ * Return true if cgroup setup mode is cgroup v1 (legacy), else
+ * returns false.
+ */
+bool is_cgroup_mode_legacy(void);
+
+/**
+ * Return true if cgroup setup mode is cgroup v1/v2 (hybrid), else
+ * returns false.
+ */
+bool is_cgroup_mode_hybrid(void);
+
+/**
+ * Return true if cgroup setup mode is cgroup v2 (unified), else
+ * returns false.
+ */
+bool is_cgroup_mode_unified(void);
+
 /**
  * @}
  * @}
