@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-2.1-only
 /*
  * Copyright RedHat Inc. 2008
  *
@@ -40,30 +41,22 @@
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * End of original copyright notice.
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2.1 of the GNU Lesser General Public License
- * as published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#include <stdio.h>
+#include <libcgroup.h>
+
+#include <syslog.h>
 #include <unistd.h>
 #include <string.h>
-#include <ctype.h>
 #include <stdlib.h>
+#include <stdio.h>
+#include <ctype.h>
 #include <errno.h>
-#include <syslog.h>
-#include <libcgroup.h>
 #include <pwd.h>
 
 /*
  * Module defines
  */
-
 #define PAM_SM_SESSION
 
 #include <security/pam_modules.h>
@@ -72,32 +65,30 @@
 #include <security/pam_ext.h>
 
 /* argument parsing */
-
 #define PAM_DEBUG_ARG       0x0001
 
 static int _pam_parse(const pam_handle_t *pamh, int argc, const char **argv)
 {
-    int ctrl = 0;
+	int ctrl = 0;
 
-    /* step through arguments */
-    for (ctrl = 0; argc-- > 0; ++argv) {
-	if (!strcmp(*argv, "debug"))
-		ctrl |= PAM_DEBUG_ARG;
-	else
-		pam_syslog(pamh, LOG_ERR, "unknown option: %s", *argv);
-    }
+	/* step through arguments */
+	for (ctrl = 0; argc-- > 0; ++argv) {
+		if (!strcmp(*argv, "debug"))
+			ctrl |= PAM_DEBUG_ARG;
+		else
+			pam_syslog(pamh, LOG_ERR, "unknown option: %s", *argv);
+	}
 
-    return ctrl;
+	return ctrl;
 }
 
 /* now the session stuff */
-PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags,
-					int argc, const char **argv)
+PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
-	pid_t pid;
-	int ctrl, ret;
-	char *user_name;
 	struct passwd *pwd;
+	char *user_name;
+	int ctrl, ret;
+	pid_t pid;
 
 	D(("called."));
 
@@ -105,16 +96,15 @@ PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags,
 
 	ret = pam_get_item(pamh, PAM_USER, (void *) &user_name);
 	if (user_name == NULL || ret != PAM_SUCCESS)  {
-		pam_syslog(pamh, LOG_ERR, "open_session - error recovering"
-				"username");
+		pam_syslog(pamh, LOG_ERR, "open_session - error recovering username");
 		return PAM_SESSION_ERR;
 	}
 
 	pwd = pam_modutil_getpwnam(pamh, user_name);
 	if (!pwd) {
 		if (ctrl & PAM_DEBUG_ARG)
-			pam_syslog(pamh, LOG_ERR, "open_session username"
-					" '%s' does not exist", user_name);
+			pam_syslog(pamh, LOG_ERR, "open_session username '%s' does not exist",
+				   user_name);
 		return PAM_SESSION_ERR;
 	}
 
@@ -124,8 +114,7 @@ PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags,
 	ret = cgroup_init();
 	if (ret) {
 		if (ctrl & PAM_DEBUG_ARG)
-			pam_syslog(pamh, LOG_ERR, "libcgroup initialization"
-							" failed");
+			pam_syslog(pamh, LOG_ERR, "libcgroup initialization failed");
 		return PAM_SESSION_ERR;
 	}
 
@@ -134,27 +123,28 @@ PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags,
 	/* Determine the pid of the task */
 	pid = getpid();
 
-	/* Note: We are using default gid here. Is there a way to determine
-	 * under what egid service will be provided?
+	/*
+	 * Note: We are using default gid here. Is there a way to
+	 * determine under what egid service will be provided?
 	 */
-	ret = cgroup_change_cgroup_uid_gid_flags(pwd->pw_uid,
-		pwd->pw_gid, pid, CGFLAG_USECACHE);
+	ret = cgroup_change_cgroup_uid_gid_flags(pwd->pw_uid, pwd->pw_gid, pid, CGFLAG_USECACHE);
 	if (ret) {
-		if (ctrl & PAM_DEBUG_ARG)
-			pam_syslog(pamh, LOG_ERR, "Change of cgroup for process"
-				" with username %s failed.\n", user_name);
+		if (ctrl & PAM_DEBUG_ARG) {
+			pam_syslog(pamh, LOG_ERR, "Change of cgroup for process with username");
+			pam_syslog(pamh, LOG_ERR, "%s failed.\n", user_name);
+		}
 		return PAM_SESSION_ERR;
 	}
 
-	if (ctrl & PAM_DEBUG_ARG)
-		pam_syslog(pamh, LOG_DEBUG, "Changed cgroup for process %d"
-				"  with username %s.\n", pid, user_name);
+	if (ctrl & PAM_DEBUG_ARG) {
+		pam_syslog(pamh, LOG_DEBUG, "Changed cgroup for process %d with username %s.\n",
+			   pid, user_name);
+	}
 
 	return PAM_SUCCESS;
 }
 
-PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags, int argc,
-					const char **argv)
+PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
 	D(("called pam_cgroup close session"));
 
