@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.7.6.  */
+/* A Bison parser, made by GNU Bison 3.8.2.  */
 
 /* Bison interface for Yacc-like parsers in C
 
@@ -62,7 +62,8 @@ extern int yydebug;
     ADMIN = 263,                   /* ADMIN  */
     NAMESPACE = 264,               /* NAMESPACE  */
     DEFAULT = 265,                 /* DEFAULT  */
-    TEMPLATE = 266                 /* TEMPLATE  */
+    TEMPLATE = 266,                /* TEMPLATE  */
+    SYSTEMD = 267                  /* SYSTEMD  */
   };
   typedef enum yytokentype yytoken_kind_t;
 #endif
@@ -80,6 +81,7 @@ extern int yydebug;
 #define NAMESPACE 264
 #define DEFAULT 265
 #define TEMPLATE 266
+#define SYSTEMD 267
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -92,7 +94,7 @@ union YYSTYPE
 	int val;
 	struct cgroup_dictionary *values;
 
-#line 96 "parse.h"
+#line 98 "parse.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
@@ -103,6 +105,8 @@ typedef union YYSTYPE YYSTYPE;
 
 extern YYSTYPE yylval;
 
+
 int yyparse (void);
 
+
 #endif /* !YY_YY_PARSE_H_INCLUDED  */
