@@ -10,13 +10,18 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <libgen.h>
 #include <stdio.h>
 
 #define MODE_SHOW_HEADERS	1
 #define MODE_SHOW_NAMES		2
+#define MODE_SYSTEMD_DELEGATE	4
 
 #define LL_MAX			100
 
+static int find_cgroup_mount_type(void);
+static int print_controller_version(void);
+
 static const struct option long_options[] = {
 	{"variable",	required_argument, NULL, 'r'},
 	{"help",	      no_argument, NULL, 'h'},
@@ -41,6 +46,11 @@ static void usage(int status, const char *program_name)
 	info("  -r, --variable <name>		Define parameter to display\n");
 	info("  -v, --values-only		Print only values, not ");
 	info("parameter names\n");
+	info("  -m				Display the cgroup mode\n");
+#ifdef WITH_SYSTEMD
+	info("  -b				Ignore default systemd delegate hierarchy\n");
+#endif
+	info("  -c				Display controller version\n");
 }
 
 static int get_controller_from_name(const char * const name, char **controller)
@@ -321,7 +331,7 @@ static int parse_opt_args(int argc, char *argv[], struct cgroup **cg_list[],
 	 */
 	if (argv[optind] == NULL && first_cg_is_dummy) {
 		usage(1, argv[0]);
-		exit(-1);
+		exit(EXIT_BADARGS);
 	}
 
 	/*
@@ -336,7 +346,7 @@ static int parse_opt_args(int argc, char *argv[], struct cgroup **cg_list[],
 	if (argv[optind] != NULL && (*cg_list_len) > 0 &&
 	    strcmp((*cg_list)[0]->name, "") != 0) {
 		usage(1, argv[0]);
-		exit(-1);
+		exit(EXIT_BADARGS);
 	}
 
 	while (argv[optind] != NULL) {
@@ -390,13 +400,23 @@ static int parse_opts(int argc, char *argv[], struct cgroup **cg_list[], int * c
 {
 	bool do_not_fill_controller = false;
 	bool first_cgroup_is_dummy = false;
+	bool cgroup_mount_type = false;
 	bool fill_controller = false;
+	bool print_ctrl_ver = false;
 	int ret = 0;
 	int c;
 
 	/* Parse arguments. */
-	while ((c = getopt_long(argc, argv, "r:hnvg:a", long_options, NULL)) > 0) {
+#ifdef WITH_SYSTEMD
+	while ((c = getopt_long(argc, argv, "r:hnvg:ambc", long_options, NULL)) > 0) {
 		switch (c) {
+		case 'b':
+			*mode = (*mode) & (INT_MAX ^ MODE_SYSTEMD_DELEGATE);
+			break;
+#else
+	while ((c = getopt_long(argc, argv, "r:hnvg:amc", long_options, NULL)) > 0) {
+		switch (c) {
+#endif
 		case 'h':
 			usage(0, argv[0]);
 			exit(0);
@@ -434,16 +454,41 @@ static int parse_opts(int argc, char *argv[], struct cgroup **cg_list[], int * c
 			if (ret)
 				goto err;
 			break;
+		case 'm':
+			cgroup_mount_type = true;
+			break;
+		case 'c':
+			print_ctrl_ver = true;
+			break;
 		default:
 			usage(1, argv[0]);
-			exit(1);
+			exit(EXIT_BADARGS);
 		}
 	}
 
 	/* Don't allow '-r' and ('-g' or '-a') */
 	if (fill_controller && do_not_fill_controller) {
 		usage(1, argv[0]);
-		exit(1);
+		exit(EXIT_BADARGS);
+	}
+
+	/* '-m' and '-c' should not used with other options */
+	if ((cgroup_mount_type || print_ctrl_ver) &&
+	    (fill_controller || do_not_fill_controller)) {
+		usage(1, argv[0]);
+		exit(EXIT_BADARGS);
+	}
+
+	if (cgroup_mount_type) {
+		ret = find_cgroup_mount_type();
+		if (ret)
+			goto err;
+	}
+
+	if (print_ctrl_ver) {
+		ret = print_controller_version();
+		if (ret)
+			goto err;
 	}
 
 	ret = parse_opt_args(argc, argv, cg_list, cg_list_len, first_cgroup_is_dummy);
@@ -458,8 +503,8 @@ static int get_cv_value(struct control_value * const cv, const char * const cg_n
 			const char * const controller_name)
 {
 	bool is_multiline = false;
+	void *tmp, *handle = NULL;
 	char tmp_line[LL_MAX];
-	void *handle, *tmp;
 	int ret;
 
 	ret = cgroup_read_value_begin(controller_name, cg_name, cv->name, &handle, tmp_line,
@@ -552,10 +597,14 @@ static int indent_multiline_value(struct control_value * const cv)
 
 static int fill_empty_controller(struct cgroup * const cg, struct cgroup_controller * const cgc)
 {
+	char cgrp_ctrl_path[FILENAME_MAX] = { '\0' };
+	char mnt_path[FILENAME_MAX] = { '\0' };
+#ifdef WITH_SYSTEMD
+	char tmp[FILENAME_MAX] = { '\0' };
+#endif
 	struct dirent *ctrl_dir = NULL;
+	int i, mnt_path_len, ret = 0;
 	bool found_mount = false;
-	int i, path_len, ret = 0;
-	char path[FILENAME_MAX];
 	DIR *dir = NULL;
 
 	pthread_rwlock_rdlock(&cg_mount_table_lock);
@@ -571,20 +620,49 @@ static int fill_empty_controller(struct cgroup * const cg, struct cgroup_control
 	if (found_mount == false)
 		goto out;
 
-	if (!cg_build_path_locked(NULL, path, cg_mount_table[i].name))
+	if (!cg_build_path_locked(NULL, mnt_path, cg_mount_table[i].name))
 		goto out;
 
-	path_len = strlen(path);
-	strncat(path, cg->name, FILENAME_MAX - path_len - 1);
-	path[sizeof(path) - 1] = '\0';
+	mnt_path_len = strlen(mnt_path);
+#ifdef WITH_SYSTEMD
+	/*
+	 * If the user has set a slice/scope as setdefault in the
+	 * cgconfig.conf file, every path constructed will have the
+	 * systemd_default_cgroup slice/scope suffixed to it.
+	 *
+	 * We need to trim the slice/scope from the path, incase of
+	 * user providing /<cgroup-name> as the cgroup name in the command
+	 * line:
+	 * cgget -g cpu:/foo
+	 */
+	if (cg->name[0] == '/' && cg->name[1] != '\0' &&
+	    strncmp(mnt_path + (mnt_path_len - 7), ".scope/", 7) == 0) {
+		snprintf(tmp, FILENAME_MAX, "%s", dirname(mnt_path));
+		strncpy(mnt_path, tmp, FILENAME_MAX - 1);
+		mnt_path[FILENAME_MAX - 1] = '\0';
 
-	if (access(path, F_OK))
+		mnt_path_len = strlen(mnt_path);
+		if (strncmp(mnt_path + (mnt_path_len - 6), ".slice", 6) == 0) {
+			snprintf(tmp, FILENAME_MAX, "%s", dirname(mnt_path));
+			strncpy(mnt_path, tmp, FILENAME_MAX - 1);
+			mnt_path[FILENAME_MAX - 1] = '\0';
+		} else {
+			cgroup_dbg("Malformed path %s (expected slice name)\n", mnt_path);
+			ret = ECGOTHER;
+			goto out;
+		}
+	}
+#endif
+	strncat(mnt_path, cg->name, FILENAME_MAX - mnt_path_len - 1);
+	mnt_path[sizeof(mnt_path) - 1] = '\0';
+
+	if (access(mnt_path, F_OK))
 		goto out;
 
-	if (!cg_build_path_locked(cg->name, path, cg_mount_table[i].name))
+	if (!cg_build_path_locked(cg->name, cgrp_ctrl_path, cg_mount_table[i].name))
 		goto out;
 
-	dir = opendir(path);
+	dir = opendir(cgrp_ctrl_path);
 	if (!dir) {
 		ret = ECGOTHER;
 		goto out;
@@ -717,7 +795,7 @@ static void print_cgroups(struct cgroup *cg_list[], int cg_list_len, int mode)
 
 int main(int argc, char *argv[])
 {
-	int mode = MODE_SHOW_NAMES | MODE_SHOW_HEADERS;
+	int mode = MODE_SHOW_NAMES | MODE_SHOW_HEADERS | MODE_SYSTEMD_DELEGATE;
 	struct cgroup **cg_list = NULL;
 	int cg_list_len = 0;
 	int ret = 0, i;
@@ -725,7 +803,7 @@ int main(int argc, char *argv[])
 	/* No parameter on input? */
 	if (argc < 2) {
 		usage(1, argv[0]);
-		return 1;
+		exit(EXIT_BADARGS);
 	}
 
 	ret = cgroup_init();
@@ -738,6 +816,10 @@ int main(int argc, char *argv[])
 	if (ret)
 		goto err;
 
+	/* this is false always for disable-systemd */
+	if (mode & MODE_SYSTEMD_DELEGATE)
+		cgroup_set_default_systemd_cgroup();
+
 	ret = get_values(cg_list, cg_list_len);
 	if (ret)
 		goto err;
@@ -750,3 +832,52 @@ err:
 
 	return ret;
 }
+
+static int find_cgroup_mount_type(void)
+{
+	enum cg_setup_mode_t setup_mode;
+	int ret = 0;
+
+	setup_mode = cgroup_setup_mode();
+	switch(setup_mode) {
+	case CGROUP_MODE_LEGACY:
+		info("Legacy Mode (Cgroup v1 only).\n");
+		break;
+	case CGROUP_MODE_HYBRID:
+		info("Hybrid mode (Cgroup v1/v2).\n");
+		break;
+	case CGROUP_MODE_UNIFIED:
+		info("Unified Mode (Cgroup v2 only).\n");
+		break;
+	default:
+		err("Unable to determine the Cgroup setup mode.\n");
+		ret = 1;
+		break;
+	}
+
+	return ret;
+}
+
+static int print_controller_version(void)
+{
+	struct cgroup_mount_point controller;
+	enum cg_version_t version;
+	void *handle;
+	int ret = 0;
+
+	/* perf_event controller is the one with the lengthiest name */
+	info("%-11s\t%-7s\n", "#Controller","Version");
+	ret = cgroup_get_controller_begin(&handle, &controller);
+	while (ret == 0) {
+		ret = cgroup_get_controller_version(controller.name, &version);
+		if (!ret)
+			info("%-11s\t%d\n", controller.name, version);
+		else
+			info("%-11s\t%-7s\n", controller.name, "unknown");
+
+		ret = cgroup_get_controller_next(&handle, &controller);
+	}
+	cgroup_get_controller_end(&handle);
+
+	return ret;
+}
