@@ -68,15 +68,26 @@ static void usage(int status, const char *program_name)
 	info("  -r, --variable <name>			Define parameter to set\n");
 	info("  --copy-from <source_cgroup_path>	Control group whose ");
 	info("parameters will be copied\n");
+#ifdef WITH_SYSTEMD
+	info("  -b					Ignore default systemd ");
+	info("delegate hierarchy\n");
+#endif
 }
 #endif /* !UNIT_TEST */
 
 STATIC int parse_r_flag(const char * const program_name, const char * const name_value_str,
 			struct control_value * const name_value)
 {
-	char *copy, *buf;
+	char *copy = NULL, *buf = NULL;
 	int ret = 0;
 
+	buf = strchr(name_value_str, '=');
+	if (buf == NULL) {
+		err("%s: wrong parameter of option -r: %s\n", program_name, optarg);
+		ret = EXIT_BADARGS;
+		goto err;
+	}
+
 	copy = strdup(name_value_str);
 	if (copy == NULL) {
 		err("%s: not enough memory\n", program_name);
@@ -88,7 +99,7 @@ STATIC int parse_r_flag(const char * const program_name, const char * const name
 	buf = strtok(copy, "=");
 	if (buf == NULL) {
 		err("%s: wrong parameter of option -r: %s\n", program_name, optarg);
-		ret = -1;
+		ret = EXIT_BADARGS;
 		goto err;
 	}
 
@@ -104,7 +115,7 @@ STATIC int parse_r_flag(const char * const program_name, const char * const name
 
 	if (strlen(buf) == 0) {
 		err("%s: wrong parameter of option -r: %s\n", program_name, optarg);
-		ret = -1;
+		ret = EXIT_BADARGS;
 		goto err;
 	}
 
@@ -121,13 +132,14 @@ err:
 #ifndef UNIT_TEST
 int main(int argc, char *argv[])
 {
+	int ignore_default_systemd_delegate_slice = 0;
 	struct control_value *name_value = NULL;
 	int nv_number = 0;
 	int nv_max = 0;
 
 	char src_cg_path[FILENAME_MAX] = "\0";
-	struct cgroup *src_cgroup;
-	struct cgroup *cgroup;
+	struct cgroup *src_cgroup = NULL;
+	struct cgroup *cgroup = NULL;
 
 	int ret = 0;
 	int c;
@@ -135,12 +147,20 @@ int main(int argc, char *argv[])
 	/* no parameter on input */
 	if (argc < 2) {
 		err("Usage is %s -r <name=value> relative path to cgroup>\n", argv[0]);
-		return -1;
+		exit(EXIT_BADARGS);
 	}
 
 	/* parse arguments */
+#ifdef WITH_SYSTEMD
+	while ((c = getopt_long (argc, argv, "r:hb", long_options, NULL)) != -1) {
+		switch (c) {
+		case 'b':
+			ignore_default_systemd_delegate_slice = 1;
+			break;
+#else
 	while ((c = getopt_long (argc, argv, "r:h", long_options, NULL)) != -1) {
 		switch (c) {
+#endif
 		case 'h':
 			usage(0, argv[0]);
 			ret = 0;
@@ -149,7 +169,7 @@ int main(int argc, char *argv[])
 		case 'r':
 			if ((flags &  FL_COPY) != 0) {
 				usage(1, argv[0]);
-				ret = -1;
+				ret = EXIT_BADARGS;
 				goto err;
 			}
 			flags |= FL_RULES;
@@ -175,7 +195,7 @@ int main(int argc, char *argv[])
 		case COPY_FROM_OPTION:
 			if (flags != 0) {
 				usage(1, argv[0]);
-				ret = -1;
+				ret = EXIT_BADARGS;
 				goto err;
 			}
 			flags |= FL_COPY;
@@ -184,7 +204,7 @@ int main(int argc, char *argv[])
 			break;
 		default:
 			usage(1, argv[0]);
-			ret = -1;
+			ret = EXIT_BADARGS;
 			goto err;
 		}
 	}
@@ -192,13 +212,13 @@ int main(int argc, char *argv[])
 	/* no cgroup name */
 	if (!argv[optind]) {
 		err("%s: no cgroup specified\n", argv[0]);
-		ret = -1;
+		ret = EXIT_BADARGS;
 		goto err;
 	}
 
 	if (flags == 0) {
 		err("%s: no name-value pair was set\n", argv[0]);
-		ret = -1;
+		ret = EXIT_BADARGS;
 		goto err;
 	}
 
@@ -209,6 +229,10 @@ int main(int argc, char *argv[])
 		goto err;
 	}
 
+	/* this is false always for disable-systemd */
+	if (!ignore_default_systemd_delegate_slice)
+		cgroup_set_default_systemd_cgroup();
+
 	/* copy the name-value pairs from -r options */
 	if ((flags & FL_RULES) != 0) {
 		src_cgroup = create_cgroup_from_name_value_pairs("tmp", name_value, nv_number);
@@ -254,7 +278,8 @@ int main(int argc, char *argv[])
 cgroup_free_err:
 	if (cgroup)
 		cgroup_free(&cgroup);
-	cgroup_free(&src_cgroup);
+	if (src_cgroup)
+		cgroup_free(&src_cgroup);
 err:
 	free(name_value);
 
