@@ -24,22 +24,22 @@
 enum flag {
 	FL_LIST =	1,
 	FL_SILENT =	2,  /* do-not show any warning/error output */
-	FL_STRICT =	4,  /* don show the variables which are not on whitelist */
+	FL_STRICT =	4,  /* don show the variables which are not on allowlist */
 	FL_OUTPUT =	8,  /* output should be redirect to the given file */
-	FL_BLACK =	16, /* blacklist set */
-	FL_WHITE =	32, /* whitelist set */
+	FL_DENY =	16, /* denylist set */
+	FL_ALLOW =	32, /* allowlist set */
 };
 
-#define BLACKLIST_CONF	"/etc/cgsnapshot_blacklist.conf"
-#define WHITELIST_CONF	"/etc/cgsnapshot_whitelist.conf"
+#define DENYLIST_CONF	"/etc/cgsnapshot_denylist.conf"
+#define ALLOWLIST_CONF	"/etc/cgsnapshot_allowlist.conf"
 
-struct black_list_type {
+struct deny_list_type {
 	char *name;			/* variable name */
-	struct black_list_type *next;	/* pointer to the next record */
+	struct deny_list_type *next;	/* pointer to the next record */
 };
 
-struct black_list_type *black_list;
-struct black_list_type *white_list;
+struct deny_list_type *deny_list;
+struct deny_list_type *allow_list;
 
 typedef char cont_name_t[FILENAME_MAX];
 
@@ -59,23 +59,23 @@ static void usage(int status, const char *program_name)
 	info("Usage: %s [-h] [-s] [-b FILE] [-w FILE] [-f FILE] [controller] [...]\n",
 	     program_name);
 	info("Generate the configuration file for given controllers\n");
-	info("  -b, --blacklist=FILE		Set the blacklist");
-	info(" configuration file (default %s)\n", BLACKLIST_CONF);
+	info("  -b, --denylist=FILE		Set the denylist");
+	info(" configuration file (default %s)\n", DENYLIST_CONF);
 	info("  -f, --file=FILE		Redirect the output to output_file\n");
 	info("  -h, --help			Display this help\n");
 	info("  -s, --silent			Ignore all warnings\n");
 	info("  -t, --strict			Don't show variables ");
-	info("which are not on the whitelist\n");
-	info("  -w, --whitelist=FILE		Set the whitelist ");
+	info("which are not on the allowlist\n");
+	info("  -w, --allowlist=FILE		Set the allowlist ");
 	info("configuration file (don't used by default)\n");
 }
 
-/* cache values from blacklist file to the list structure */
-int load_list(char *filename, struct black_list_type **p_list)
+/* cache values from denylist file to the list structure */
+int load_list(char *filename, struct deny_list_type **p_list)
 {
-	struct black_list_type *start = NULL;
-	struct black_list_type *end = NULL;
-	struct black_list_type *new;
+	struct deny_list_type *start = NULL;
+	struct deny_list_type *end = NULL;
+	struct deny_list_type *new;
 
 	char buf[FILENAME_MAX];
 	char name[FILENAME_MAX];
@@ -106,7 +106,7 @@ int load_list(char *filename, struct black_list_type **p_list)
 		if (ret == 0)
 			continue;
 
-		new = (struct black_list_type *) malloc(sizeof(struct black_list_type));
+		new = (struct deny_list_type *) malloc(sizeof(struct deny_list_type));
 		if (new == NULL) {
 			err("ERROR: Memory allocation problem (%s)\n", strerror(errno));
 			ret = 1;
@@ -152,10 +152,10 @@ err:
 }
 
 /* free list structure */
-void free_list(struct black_list_type *list)
+void free_list(struct deny_list_type *list)
 {
-	struct black_list_type *now;
-	struct black_list_type *next;
+	struct deny_list_type *now;
+	struct deny_list_type *next;
 
 	now = list;
 	while (now != NULL) {
@@ -171,9 +171,9 @@ void free_list(struct black_list_type *list)
  * 1 ... was found
  * 0 ... no record was found
  */
-int is_on_list(char *name, struct black_list_type *list)
+int is_on_list(char *name, struct deny_list_type *list)
 {
-	struct black_list_type *record;
+	struct deny_list_type *record;
 
 	record = list;
 	/* go through the list of all values */
@@ -236,12 +236,14 @@ static int display_permissions(const char *path, const char * const cg_name,
 		pw = getpwuid(sba.st_uid);
 		if (pw == NULL) {
 			err("ERROR: can't get %d user name\n", sba.st_uid);
+			fprintf(output_f, "}\n}\n");
 			return -1;
 		}
 
 		gr = getgrgid(sba.st_gid);
 		if (gr == NULL) {
 			err("ERROR: can't get %d group name\n", sba.st_gid);
+			fprintf(output_f, "}\n}\n");
 			return -1;
 		}
 
@@ -255,12 +257,14 @@ static int display_permissions(const char *path, const char * const cg_name,
 		pw = getpwuid(sbt.st_uid);
 		if (pw == NULL) {
 			err("ERROR: can't get %d user name\n", sbt.st_uid);
+			fprintf(output_f, "}\n}\n");
 			return -1;
 		}
 
 		gr = getgrgid(sbt.st_gid);
 		if (gr == NULL) {
 			err("ERROR: can't get %d group name\n", sbt.st_gid);
+			fprintf(output_f, "}\n}\n");
 			return -1;
 		}
 
@@ -293,7 +297,7 @@ static int display_cgroup_data(struct cgroup *group,
 	char *value = NULL;
 	char *output_name;
 	struct stat sb;
-	int bl, wl = 0; /* is on the blacklist/whitelist flag */
+	int bl, wl = 0; /* is on the denylist/allowlist flag */
 	int nr_var = 0;
 	int i = 0, j;
 	int ret = 0;
@@ -363,30 +367,30 @@ static int display_cgroup_data(struct cgroup *group,
 			}
 
 			/*
-			 * find whether the variable is blacklisted
-			 * or whitelisted
+			 * find whether the variable is denylisted
+			 * or allowlisted
 			 */
-			bl = is_on_list(name, black_list);
-			wl = is_on_list(name, white_list);
+			bl = is_on_list(name, deny_list);
+			wl = is_on_list(name, allow_list);
 
-			/* if it is blacklisted skip it and continue */
+			/* if it is denylisted skip it and continue */
 			if (bl)
 				continue;
 
 			/*
-			 * if it is not whitelisted and strict tag is used
+			 * if it is not allowlisted and strict tag is used
 			 * skip it and continue too
 			 */
 			if ((!wl) && (flags &  FL_STRICT))
 				continue;
 
 			/*
-			 * if it is not whitelisted and silent tag is not
+			 * if it is not allowlisted and silent tag is not
 			 * used write an warning
 			 */
 			if ((!wl) && !(flags &  FL_SILENT) && (first)) {
 				err("WARNING: variable %s is neither ", name);
-				err("blacklisted nor whitelisted\n");
+				err("deny nor allow list\n");
 			}
 
 			output_name = name;
@@ -429,7 +433,7 @@ err:
 }
 
 /*
- * creates the record about the hierarchie which contains
+ * creates the record about the hierarchies which contains
  * "controller" subsystem
  */
 static int display_controller_data(char controller[CG_CONTROLLER_MAX][FILENAME_MAX],
@@ -472,13 +476,22 @@ static int display_controller_data(char controller[CG_CONTROLLER_MAX][FILENAME_M
 
 			ret = cgroup_get_cgroup(group);
 			if (ret != 0) {
-				info("cannot read group '%s': %s\n", cgroup_name,
-				     cgroup_strerror(ret));
-				goto err;
+				/*
+				 * We know for sure that the cgroup exists
+				 * but just that the cgroup v2 controller
+				 * is not enabled in the cgroup.subtree_control
+				 * file.
+				 */
+				if (ret != ECGROUPNOTEXIST) {
+					info("cannot read group '%s': %s %d\n", cgroup_name,
+					     cgroup_strerror(ret), ret);
+					goto err;
+				}
 			}
 
-			display_cgroup_data(group, controller, info.full_path, prefix_len, first,
-					    program_name);
+			if (ret == 0)
+				display_cgroup_data(group, controller, info.full_path, prefix_len,
+						    first, program_name);
 			first = 0;
 			cgroup_free(&group);
 		}
@@ -493,22 +506,40 @@ err:
 }
 
 static int is_ctlr_on_list(char controllers[CG_CONTROLLER_MAX][FILENAME_MAX],
-			   cont_name_t wanted_conts[FILENAME_MAX])
+			   cont_name_t wanted_conts[CG_CONTROLLER_MAX])
 {
-	int i = 0;
-	int j = 0;
+	char tmp_controllers[CG_CONTROLLER_MAX][FILENAME_MAX];
+	int i = 0, j = 0, k = 0;
+	int ret = 0;
 
 	while (controllers[i][0] != '\0') {
 		while (wanted_conts[j][0] != '\0') {
-			if (strcmp(controllers[i], wanted_conts[j]) == 0)
-				return 1;
+			if (strcmp(controllers[i], wanted_conts[j]) == 0) {
+				strncpy(tmp_controllers[k], wanted_conts[j], FILENAME_MAX - 1);
+				(tmp_controllers[k])[FILENAME_MAX - 1] = '\0';
+				k++;
+			}
 			j++;
 		}
 		j = 0;
 		i++;
 	}
 
-	return 0;
+	(tmp_controllers[k])[0] = '\0';
+
+	/* Lets reset the controllers to intersection of controller ∩ wanted_conts */
+	for (i = 0; tmp_controllers[i][0] != '\0'; i++) {
+		/*
+		 * gcc complains about truncation when using snprintf() and
+		 * and coverity complains about truncation when using strncpy().
+		 * Avoid both these warnings by directly invoking memcpy()
+		 */
+		memcpy(controllers[i], tmp_controllers[i], sizeof(controllers[i]));
+		ret = 1;
+	}
+	(controllers[i])[0] = '\0';
+
+	return ret;
 }
 
 
@@ -529,8 +560,13 @@ static int parse_controllers(cont_name_t cont_names[CG_CONTROLLER_MAX], const ch
 	/* go through the list of controllers/mount point pairs */
 	while (ret == 0) {
 		if (strcmp(path, controller.path) == 0) {
-			/* if it is still the same mount point */
-			if (max < CG_CONTROLLER_MAX) {
+			/*
+			 * if it is still the same mount point
+			 *
+			 * note that the last entry in controllers[][] must be '\0', so
+			 * we need to stop populating the array at CG_CONTROLLER_MAX - 1
+			 */
+			if (max < CG_CONTROLLER_MAX - 1) {
 				strncpy(controllers[max], controller.name, FILENAME_MAX);
 				(controllers[max])[FILENAME_MAX-1] = '\0';
 				max++;
@@ -695,16 +731,16 @@ int main(int argc, char *argv[])
 	static struct option long_opts[] = {
 		{"help",	      no_argument, NULL, 'h'},
 		{"silent",	      no_argument, NULL, 's'},
-		{"blacklist",	required_argument, NULL, 'b'},
-		{"whitelist",	required_argument, NULL, 'w'},
+		{"denylist",	required_argument, NULL, 'b'},
+		{"allowlist",	required_argument, NULL, 'w'},
 		{"strict",	      no_argument, NULL, 't'},
 		{"file",	required_argument, NULL, 'f'},
 		{0, 0, 0, 0}
 	};
 
 	cont_name_t wanted_cont[CG_CONTROLLER_MAX];
-	char bl_file[FILENAME_MAX];  /* blacklist file name */
-	char wl_file[FILENAME_MAX];  /* whitelist file name */
+	char bl_file[FILENAME_MAX];  /* denylist file name */
+	char wl_file[FILENAME_MAX];  /* allowlist file name */
 	int ret = 0, err;
 	int c_number = 0;
 	int c, i;
@@ -724,12 +760,12 @@ int main(int argc, char *argv[])
 			flags |= FL_SILENT;
 			break;
 		case 'b':
-			flags |= FL_BLACK;
+			flags |= FL_DENY;
 			strncpy(bl_file, optarg, FILENAME_MAX);
 			bl_file[FILENAME_MAX-1] = '\0';
 			break;
 		case 'w':
-			flags |= FL_WHITE;
+			flags |= FL_ALLOW;
 			strncpy(wl_file, optarg, FILENAME_MAX);
 			wl_file[FILENAME_MAX-1] = '\0';
 			break;
@@ -746,7 +782,7 @@ int main(int argc, char *argv[])
 			break;
 		default:
 			usage(1, argv[0]);
-			return -1;
+			exit(EXIT_BADARGS);
 		}
 	}
 
@@ -766,21 +802,25 @@ int main(int argc, char *argv[])
 	if ((flags & FL_OUTPUT) == 0)
 		output_f = stdout;
 
-	/* blacklkist */
-	if (flags & FL_BLACK) {
-		ret  = load_list(bl_file, &black_list);
+	/* denylist */
+	if (flags & FL_DENY) {
+		ret  = load_list(bl_file, &deny_list);
 	} else {
-		/* load the blacklist from the default location */
-		ret  = load_list(BLACKLIST_CONF, &black_list);
+		/* load the denylist from the default location */
+		ret  = load_list(DENYLIST_CONF, &deny_list);
 	}
-	if (ret != 0)
+	if (ret != 0) {
+		ret = EXIT_BADARGS;
 		goto finish;
+	}
 
-	/* whitelist */
-	if (flags & FL_WHITE)
-		ret = load_list(wl_file, &white_list);
-	if (ret != 0)
+	/* allowlist */
+	if (flags & FL_ALLOW)
+		ret = load_list(wl_file, &allow_list);
+	if (ret != 0) {
+		ret = EXIT_BADARGS;
 		goto finish;
+	}
 
 	/* print the header */
 	fprintf(output_f, "# Configuration file generated by cgsnapshot\n");
@@ -793,6 +833,7 @@ int main(int argc, char *argv[])
 
 	/* print mount points section */
 	ret = parse_mountpoints(wanted_cont, argv[0]);
+	ret = ret ? EXIT_BADARGS : 0;
 	/* continue with processing on error*/
 
 	/* print hierarchies section */
@@ -802,8 +843,8 @@ int main(int argc, char *argv[])
 		ret = err;
 
 finish:
-	free_list(black_list);
-	free_list(white_list);
+	free_list(deny_list);
+	free_list(allow_list);
 
 	if (output_f != stdout)
 		fclose(output_f);
