@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1-only
 /**
- * Libcgroup extended cgget.  Works with both cgroup v1 and v2
+ * Libcgroup extended cgxget.  Works with both cgroup v1 and v2
  *
  * Copyright (c) 2021-2022 Oracle and/or its affiliates.
  * Author: Tom Hromatka <tom.hromatka@oracle.com>
@@ -17,11 +17,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <libgen.h>
 #include <stdio.h>
 
 
 #define MODE_SHOW_HEADERS	1
 #define MODE_SHOW_NAMES		2
+#define MODE_SYSTEMD_DELEGATE	4
 
 #define LL_MAX			100
 
@@ -58,6 +60,9 @@ static void usage(int status, const char *program_name)
 	info("  -n				Do not print headers\n");
 	info("  -r, --variable  <name>	Define parameter to display\n");
 	info("  -v, --values-only		Print only values, not parameter names\n");
+#ifdef WITH_SYSTEMD
+	info("  -b				Ignore default systemd delegate hierarchy\n");
+#endif
 }
 
 static int get_controller_from_name(const char * const name, char **controller)
@@ -70,7 +75,7 @@ static int get_controller_from_name(const char * const name, char **controller)
 
 	dot = strchr(*controller, '.');
 	if (dot == NULL) {
-		err("cgget: error parsing parameter name\n '%s'", name);
+		err("cgxget: error parsing parameter name\n '%s'", name);
 		return ECGINVAL;
 	}
 	*dot = '\0';
@@ -175,7 +180,7 @@ static int parse_r_flag(struct cgroup **cg_list[], int * const cg_list_len,
 	if (!cgc) {
 		cgc = cgroup_add_controller(cg, cntl_value_controller);
 		if (!cgc) {
-			err("cgget: cannot find controller '%s'\n", cntl_value_controller);
+			err("cgxget: cannot find controller '%s'\n", cntl_value_controller);
 			ret = ECGOTHER;
 			goto out;
 		}
@@ -337,7 +342,7 @@ static int parse_opt_args(int argc, char *argv[], struct cgroup **cg_list[],
 	 */
 	if (argv[optind] == NULL && first_cg_is_dummy) {
 		usage(1, argv[0]);
-		exit(-1);
+		exit(EXIT_BADARGS);
 	}
 
 	/*
@@ -347,11 +352,11 @@ static int parse_opt_args(int argc, char *argv[], struct cgroup **cg_list[],
 	 * cgget implementation.  Continue that approach.
 	 *
 	 * Example of a command that will hit this code:
-	 *	$ cgget -g cpu:mycgroup mycgroup
+	 *	$ cgxget -g cpu:mycgroup mycgroup
 	 */
 	if (argv[optind] != NULL && (*cg_list_len) > 0 && strcmp((*cg_list)[0]->name, "") != 0) {
 		usage(1, argv[0]);
-		exit(-1);
+		exit(EXIT_BADARGS);
 	}
 
 	while (argv[optind] != NULL) {
@@ -412,8 +417,16 @@ static int parse_opts(int argc, char *argv[], struct cgroup **cg_list[], int * c
 	int c;
 
 	/* Parse arguments. */
+#ifdef WITH_SYSTEMD
+	while ((c = getopt_long(argc, argv, "r:hnvg:a12ib", long_options, NULL)) > 0) {
+		switch (c) {
+		case 'b':
+			*mode = (*mode) & (INT_MAX ^ MODE_SYSTEMD_DELEGATE);
+			break;
+#else
 	while ((c = getopt_long(argc, argv, "r:hnvg:a12i", long_options, NULL)) > 0) {
 		switch (c) {
+#endif
 		case 'h':
 			usage(0, argv[0]);
 			exit(0);
@@ -462,14 +475,14 @@ static int parse_opts(int argc, char *argv[], struct cgroup **cg_list[], int * c
 			break;
 		default:
 			usage(1, argv[0]);
-			exit(1);
+			exit(EXIT_BADARGS);
 		}
 	}
 
 	/* Don't allow '-r' and ('-g' or '-a') */
 	if (fill_controller && do_not_fill_controller) {
 		usage(1, argv[0]);
-		exit(1);
+		exit(EXIT_BADARGS);
 	}
 
 	ret = parse_opt_args(argc, argv, cg_list, cg_list_len, first_cgroup_is_dummy);
@@ -504,7 +517,7 @@ static int get_cv_value(struct control_value * const cv, const char * const cg_n
 			 */
 			tmp_ret = cgroup_test_subsys_mounted(controller_name);
 			if (tmp_ret == 0) {
-				err("cgget: cannot find controller '%s' in group '%s'\n",
+				err("cgxget: cannot find controller '%s' in group '%s'\n",
 				    controller_name, cg_name);
 			} else {
 				err("variable file read failed %s\n", cgroup_strerror(ret));
@@ -576,10 +589,14 @@ static int indent_multiline_value(struct control_value * const cv)
 
 static int fill_empty_controller(struct cgroup * const cg, struct cgroup_controller * const cgc)
 {
+	char cgrp_ctrl_path[FILENAME_MAX] = { '\0' };
+	char mnt_path[FILENAME_MAX] = { '\0' };
+#ifdef WITH_SYSTEMD
+	char tmp[FILENAME_MAX] = { '\0' };
+#endif
 	struct dirent *ctrl_dir = NULL;
-	char path[FILENAME_MAX];
+	int i, mnt_path_len, ret = 0;
 	bool found_mount = false;
-	int i, path_len, ret = 0;
 	DIR *dir = NULL;
 
 	pthread_rwlock_rdlock(&cg_mount_table_lock);
@@ -597,21 +614,50 @@ static int fill_empty_controller(struct cgroup * const cg, struct cgroup_control
 	if (found_mount == false)
 		goto out;
 
-	if (!cg_build_path_locked(NULL, path, cg_mount_table[i].name))
+	if (!cg_build_path_locked(NULL, mnt_path, cg_mount_table[i].name))
 		goto out;
 
-	path_len = strlen(path);
-	strncat(path, cg->name, FILENAME_MAX - path_len - 1);
-	path[sizeof(path) - 1] = '\0';
+	mnt_path_len = strlen(mnt_path);
+#ifdef WITH_SYSTEMD
+		/*
+		 * If the user has set a slice/scope as setdefault in the
+		 * cgconfig.conf file, every path constructed will have the
+		 * systemd_default_cgroup slice/scope suffixed to it.
+		 *
+		 * We need to trim the slice/scope from the path, incase of
+		 * user providing /<cgroup-name> as the cgroup name in the command
+		 * line:
+		 * cgxget -1  -r cpu.shares /a
+		 */
 
-	if (access(path, F_OK))
+	if (cg->name[0] == '/' && cg->name[1] != '\0' &&
+	    strncmp(mnt_path + (mnt_path_len - 7), ".scope/", 7) == 0) {
+		snprintf(tmp, FILENAME_MAX, "%s", dirname(mnt_path));
+		strncpy(mnt_path, tmp, FILENAME_MAX - 1);
+		mnt_path[FILENAME_MAX - 1] = '\0';
+
+		mnt_path_len = strlen(mnt_path);
+		if (strncmp(mnt_path + (mnt_path_len - 6), ".slice", 6) == 0) {
+			snprintf(tmp, FILENAME_MAX, "%s", dirname(mnt_path));
+			strncpy(mnt_path, tmp, FILENAME_MAX - 1);
+			mnt_path[FILENAME_MAX - 1] = '\0';
+		} else {
+			cgroup_dbg("Malformed path %s (expected slice name)\n", mnt_path);
+			ret  = ECGOTHER;
+			goto out;
+		}
+	}
+#endif
+	strncat(mnt_path, cg->name, FILENAME_MAX - mnt_path_len - 1);
+	mnt_path[sizeof(mnt_path) - 1] = '\0';
+
+	if (access(mnt_path, F_OK))
 		goto out;
 
-	if (!cg_build_path_locked(cg->name, path,
-		cg_mount_table[i].name))
+	if (!cg_build_path_locked(cg->name, cgrp_ctrl_path, cg_mount_table[i].name))
 		goto out;
 
-	dir = opendir(path);
+	dir = opendir(cgrp_ctrl_path);
 	if (!dir) {
 		ret = ECGOTHER;
 		goto out;
@@ -777,7 +823,7 @@ out:
 		 * Free the old list.
 		 */
 		for (i = 0; i < cg_list_len; i++)
-			cgroup_free(cg_list[i]);
+			cgroup_free(&(*cg_list)[i]);
 
 		*cg_list = cg_converted_list;
 	}
@@ -787,7 +833,7 @@ out:
 
 int main(int argc, char *argv[])
 {
-	int mode = MODE_SHOW_NAMES | MODE_SHOW_HEADERS;
+	int mode = MODE_SHOW_NAMES | MODE_SHOW_HEADERS | MODE_SYSTEMD_DELEGATE;
 	enum cg_version_t version = CGROUP_UNK;
 	struct cgroup **cg_list = NULL;
 	bool ignore_unmappable = false;
@@ -797,7 +843,7 @@ int main(int argc, char *argv[])
 	/* No parameter on input? */
 	if (argc < 2) {
 		usage(1, argv[0]);
-		return 1;
+		exit(EXIT_BADARGS);
 	}
 
 	ret = cgroup_init();
@@ -810,14 +856,18 @@ int main(int argc, char *argv[])
 	if (ret)
 		goto err;
 
+	/* this is false always for disable-systemd */
+	if (mode & MODE_SYSTEMD_DELEGATE)
+		cgroup_set_default_systemd_cgroup();
+
 	ret = convert_cgroups(&cg_list, cg_list_len, version, CGROUP_DISK);
-	if (ret == ECGNOVERSIONCONVERT && ignore_unmappable)
+	if ((ret && ret != ECGNOVERSIONCONVERT) ||
+	    (ret == ECGNOVERSIONCONVERT && !ignore_unmappable))
 		/*
-		 * The user has specified that we should ignore any errors
-		 * due to being unable to map from v1 to v2 or vice versa
+		 * If the user not has specified that we ignore any errors
+		 * due to being unable to map from v1 to v2 or vice versa,
+		 * return error, else ignore the error and continue.
 		 */
-		ret = 0;
-	else if (ret)
 		goto err;
 
 	ret = get_values(cg_list, cg_list_len);
