@@ -23,10 +23,13 @@ extern "C" {
 #define cgroup_warn(x...)	cgroup_log(CGROUP_LOG_WARNING, "Warning: " x)
 #define cgroup_info(x...)	cgroup_log(CGROUP_LOG_INFO, "Info: " x)
 #define cgroup_dbg(x...)	cgroup_log(CGROUP_LOG_DEBUG, x)
+#define cgroup_cont(x...)	cgroup_log(CGROUP_LOG_CONT, x)
 
 #define err(x...)	fprintf(stderr, x)
 #define info(x...)	fprintf(stdout, x)
 
+#define EXIT_BADARGS	129
+
 /**
  * Auxiliary specifier of group, used to store parsed command line options.
  */
