@@ -15,10 +15,7 @@ import os
 
 
 def prereqs(config):
-    result = consts.TEST_PASSED
-    cause = None
-
-    return result, cause
+    pass
 
 
 def setup(config):
@@ -66,7 +63,6 @@ def test(config):
     if 'Usage:' not in ret:
         result = consts.TEST_FAILED
         cause = 'Failed to print help text'
-        return result, cause
 
     return result, cause
 
@@ -76,9 +72,7 @@ def teardown(config):
 
 
 def main(config):
-    [result, cause] = prereqs(config)
-    if result != consts.TEST_PASSED:
-        return [result, cause]
+    prereqs(config)
 
     try:
         setup(config)
