@@ -7,8 +7,8 @@
 # Author: Tom Hromatka <tom.hromatka@oracle.com>
 #
 
+from process import Process
 from cgroup import Cgroup
-from run import Run
 import consts
 import ftests
 import sys
@@ -20,12 +20,13 @@ CGNAME = '034cgexec'
 
 
 def prereqs(config):
+    result = consts.TEST_PASSED
+    cause = None
     if not config.args.container:
         result = consts.TEST_SKIPPED
         cause = 'This test must be run within a container'
-        return result, cause
 
-    return consts.TEST_PASSED, None
+    return result, cause
 
 
 def setup(config):
@@ -33,6 +34,9 @@ def setup(config):
 
 
 def test(config):
+    result = consts.TEST_PASSED
+    cause = None
+
     config.process.create_process_in_cgroup(config, CONTROLLER, CGNAME,
                                             cgclassify=False)
 
@@ -48,19 +52,13 @@ def test(config):
     if 'Run the task in given control group(s)' not in ret:
         result = consts.TEST_FAILED
         cause = 'Failed to print cgexec help text: {}'.format(ret)
-        return result, cause
 
-    return consts.TEST_PASSED, None
+    return result, cause
 
 
 def teardown(config):
     pids = Cgroup.get_pids_in_cgroup(config, CGNAME, CONTROLLER)
-    if pids:
-        for p in pids.splitlines():
-            if config.args.container:
-                config.container.run(['kill', '-9', p])
-            else:
-                Run.run(['sudo', 'kill', '-9', p])
+    Process.kill(config, pids)
 
     Cgroup.delete(config, CONTROLLER, CGNAME)
 
