@@ -2,7 +2,7 @@
 #
 # Cgroup class for the libcgroup functional tests
 #
-# Copyright (c) 2019-2021 Oracle and/or its affiliates.
+# Copyright (c) 2019-2022 Oracle and/or its affiliates.
 # Author: Tom Hromatka <tom.hromatka@oracle.com>
 #
 
@@ -10,6 +10,7 @@ from container import ContainerError
 from controller import Controller
 from run import Run, RunError
 import multiprocessing as mp
+from libcgroup import Mode
 from enum import Enum
 import consts
 import utils
@@ -19,7 +20,7 @@ import os
 
 
 class CgroupMount(object):
-    def __init__(self, mount_line):
+    def __init__(self, mount_line, controller=None):
         entries = mount_line.split()
 
         if entries[2] == 'cgroup':
@@ -31,14 +32,17 @@ class CgroupMount(object):
 
         self.mount_point = entries[1]
 
-        self.controller = None
-        if self.version == CgroupVersion.CGROUP_V1:
-            self.controller = entries[3].split(',')[-1]
+        if controller:
+            self.controller = controller
+        else:
+            self.controller = None
+            if self.version == CgroupVersion.CGROUP_V1:
+                self.controller = entries[3].split(',')[-1]
 
-            if self.controller == 'clone_children':
-                # the cpuset controller may append this option to the end
-                # rather than the controller name like all other controllers
-                self.controller = 'cpuset'
+                if self.controller == 'clone_children':
+                    # the cpuset controller may append this option to the end
+                    # rather than the controller name like all other controllers
+                    self.controller = 'cpuset'
 
     def __str__(self):
         out_str = 'CgroupMount'
@@ -122,7 +126,9 @@ class Cgroup(object):
     @staticmethod
     def create(config, controller_list, cgname, user_name=None,
                group_name=None, dperm=None, fperm=None, tperm=None,
-               tasks_user_name=None, tasks_group_name=None, cghelp=False):
+               tasks_user_name=None, tasks_group_name=None, cghelp=False,
+               ignore_systemd=False, create_scope=False, set_default_scope=False,
+               scope_pid=-1):
         if isinstance(controller_list, str):
             controller_list = [controller_list]
 
@@ -155,6 +161,26 @@ class Cgroup(object):
         if cghelp:
             cmd.append('-h')
 
+        # -b and -c are mutually exclusive but the safety check is done in libcgroup
+        # itself and thus doesn't need to be done here
+
+        if ignore_systemd:
+            cmd.append('-b')
+
+        if create_scope:
+            cmd.append('-c')
+
+        # -S requires -c to be provided, but this is checked in libcgroup itself
+
+        if set_default_scope:
+            cmd.append('-S')
+
+        # -p requires -c to be provided, but this is too checked in libcgroup itself
+
+        if scope_pid > 1:
+            cmd.append('-p')
+            cmd.append('{}'.format(scope_pid))
+
         if controller_list:
             controllers_and_path = '{}:{}'.format(
                 ','.join(controller_list), cgname)
@@ -167,10 +193,28 @@ class Cgroup(object):
         if config.args.container:
             config.container.run(cmd)
         else:
-            Run.run(cmd)
+            if create_scope:
+                # creating a scope causes libcgroup to create a child process.
+                # subprocess.Popen.communicate() will not return until the created
+                # process and all of its children complete.  Get around this by
+                # timing out the operation
+                Run.run(cmd, timeout=5)
+            else:
+                Run.run(cmd)
+
+    # This is a simple wrapper, that calls Cgroup.create() to create the
+    # cgroup and cgroup_exists(), which builds the cgroup path and returns
+    # True if the cgroup exists, False otherwise
+    @staticmethod
+    def create_and_validate(config, ctrl_name, cgroup_name, ignore_systemd=False,
+                            create_scope=False, set_default_scope=False, scope_pid=-1):
+        Cgroup.create(config, ctrl_name, cgroup_name, ignore_systemd=ignore_systemd,
+                      create_scope=create_scope, set_default_scope=set_default_scope,
+                      scope_pid=scope_pid)
+        return Cgroup.exists(config, ctrl_name, cgroup_name, ignore_systemd=ignore_systemd)
 
     @staticmethod
-    def delete(config, controller_list, cgname, recursive=False):
+    def delete(config, controller_list, cgname, recursive=False, ignore_systemd=False):
         if isinstance(controller_list, str):
             controller_list = [controller_list]
 
@@ -192,6 +236,9 @@ class Cgroup(object):
         cmd.append('-g')
         cmd.append(controllers_and_path)
 
+        if ignore_systemd:
+            cmd.append('-b')
+
         if config.args.container:
             config.container.run(cmd)
         else:
@@ -199,11 +246,11 @@ class Cgroup(object):
 
     @staticmethod
     def __set(config, cmd, cgname=None, setting=None, value=None,
-              copy_from=None, cghelp=False):
+              copy_from=None, cghelp=False, ignore_systemd=False):
         if setting is not None or value is not None:
-            if isinstance(setting, str) and isinstance(value, str):
+            if isinstance(setting, str) and (isinstance(value, str) or isinstance(value, int)):
                 cmd.append('-r')
-                cmd.append('{}={}'.format(setting, value))
+                cmd.append('{}={}'.format(setting, str(value)))
             elif isinstance(setting, list) and isinstance(value, list):
                 if len(setting) != len(value):
                     raise ValueError(
@@ -213,7 +260,7 @@ class Cgroup(object):
 
                 for idx, stg in enumerate(setting):
                     cmd.append('-r')
-                    cmd.append('{}={}'.format(stg, value[idx]))
+                    cmd.append('{}={}'.format(stg, str(value[idx])))
             else:
                 raise ValueError(
                                     'Invalid inputs to cgget:\nsetting: {}\n'
@@ -233,6 +280,9 @@ class Cgroup(object):
                 for cg in cgname:
                     cmd.append(cg)
 
+        if ignore_systemd:
+            cmd.append('-b')
+
         if cghelp:
             cmd.append('-h')
 
@@ -243,7 +293,7 @@ class Cgroup(object):
 
     @staticmethod
     def set(config, cgname=None, setting=None, value=None, copy_from=None,
-            cghelp=False):
+            cghelp=False, ignore_systemd=False):
         """cgset equivalent method
 
         The following variants of cgset are being tested by the
@@ -265,12 +315,12 @@ class Cgroup(object):
         cmd.append(Cgroup.build_cmd_path('cgset'))
 
         return Cgroup.__set(config, cmd, cgname, setting, value, copy_from,
-                            cghelp)
+                            cghelp, ignore_systemd)
 
     @staticmethod
     def xset(config, cgname=None, setting=None, value=None, copy_from=None,
              version=CgroupVersion.CGROUP_UNK, cghelp=False,
-             ignore_unmappable=False):
+             ignore_unmappable=False, ignore_systemd=False):
         """cgxset equivalent method
         """
         cmd = list()
@@ -287,17 +337,21 @@ class Cgroup(object):
             cmd.append('-i')
 
         return Cgroup.__set(config, cmd, cgname, setting, value, copy_from,
-                            cghelp)
+                            cghelp, ignore_systemd)
 
     @staticmethod
     def __get(config, cmd, controller=None, cgname=None, setting=None,
               print_headers=True, values_only=False,
-              all_controllers=False, cghelp=False):
+              all_controllers=False, cghelp=False, ignore_systemd=False):
         if not print_headers:
             cmd.append('-n')
+
         if values_only:
             cmd.append('-v')
 
+        if ignore_systemd:
+            cmd.append('-b')
+
         if setting is not None:
             if isinstance(setting, str):
                 # the user provided a simple string.  use it as is
@@ -350,7 +404,7 @@ class Cgroup(object):
     @staticmethod
     def get(config, controller=None, cgname=None, setting=None,
             print_headers=True, values_only=False,
-            all_controllers=False, cghelp=False):
+            all_controllers=False, cghelp=False, ignore_systemd=False):
         """cgget equivalent method
 
         Returns:
@@ -376,13 +430,13 @@ class Cgroup(object):
 
         return Cgroup.__get(config, cmd, controller, cgname, setting,
                             print_headers, values_only, all_controllers,
-                            cghelp)
+                            cghelp, ignore_systemd)
 
     @staticmethod
     def xget(config, controller=None, cgname=None, setting=None,
              print_headers=True, values_only=False,
              all_controllers=False, version=CgroupVersion.CGROUP_UNK,
-             cghelp=False, ignore_unmappable=False):
+             cghelp=False, ignore_unmappable=False, ignore_systemd=False):
         """cgxget equivalent method
 
         Returns:
@@ -401,18 +455,35 @@ class Cgroup(object):
 
         return Cgroup.__get(config, cmd, controller, cgname, setting,
                             print_headers, values_only, all_controllers,
-                            cghelp)
+                            cghelp, ignore_systemd)
 
     @staticmethod
     def classify(config, controller, cgname, pid_list, sticky=False,
-                 cancel_sticky=False):
+                 cancel_sticky=False, ignore_systemd=False, replace_idle=False):
         cmd = list()
 
         if not config.args.container:
             cmd.append('sudo')
         cmd.append(Cgroup.build_cmd_path('cgclassify'))
-        cmd.append('-g')
-        cmd.append('{}:{}'.format(controller, cgname))
+
+        if replace_idle:
+            cmd.append('-r')
+
+        if ignore_systemd:
+            cmd.append('-b')
+
+        if isinstance(controller, str):
+            cmd.append('-g')
+            cmd.append('{}:{}'.format(controller, cgname))
+        elif isinstance(controller, list):
+            controller = ','.join(controller)
+            cmd.append('-g')
+            cmd.append('{}:{}'.format(controller, cgname))
+        elif controller is None:
+            cmd.append('-g')
+            cmd.append(':{}'.format(cgname))
+        else:
+            raise ValueError('Unsupported controller format: {}'.format(type(controller)))
 
         if isinstance(pid_list, str):
             cmd.append(pid_list)
@@ -420,7 +491,7 @@ class Cgroup(object):
             cmd.append(str(pid_list))
         elif isinstance(pid_list, list):
             for pid in pid_list:
-                cmd.append(pid)
+                cmd.append(str(pid))
 
         if config.args.container:
             config.container.run(cmd)
@@ -533,13 +604,13 @@ class Cgroup(object):
                 config.container.run(
                                         ['sudo',
                                          'touch',
-                                         '/etc/cgsnapshot_blacklist.conf']
+                                         '/etc/cgsnapshot_denylist.conf']
                                     )
             except RunError as re:
                 if re.ret == 0 and 'unable to resolve host' in re.stderr:
                     pass
         else:
-            Run.run(['sudo', 'touch', '/etc/cgsnapshot_blacklist.conf'])
+            Run.run(['sudo', 'touch', '/etc/cgsnapshot_denylist.conf'])
 
         try:
             if config.args.container:
@@ -548,7 +619,9 @@ class Cgroup(object):
                 res = Run.run(cmd)
         except RunError as re:
             if re.ret == 0 and \
-               'neither blacklisted nor whitelisted' in re.stderr:
+               'neither deny nor allow list' in re.stderr:
+                res = re.stdout
+            elif re.ret == 0 and 'ERROR: can\'t get' in re.stderr:
                 res = re.stdout
             else:
                 raise(re)
@@ -719,6 +792,10 @@ class Cgroup(object):
             for line in mntf.readlines():
                 mnt_path = line.split()[1]
 
+                if ctrl_name is None:
+                    if line.split()[0] == 'cgroup2':
+                        return mnt_path
+
                 if line.split()[0] == 'cgroup':
                     for option in line.split()[3].split(','):
                         if option == ctrl_name:
@@ -736,6 +813,9 @@ class Cgroup(object):
                 mnt_path = line.split()[1]
 
                 if line.split()[0] == 'cgroup2':
+                    if ctrl_name is None:
+                        return mnt_path
+
                     ctrl_file = os.path.join(line.split()[1],
                                              'cgroup.controllers')
 
@@ -752,7 +832,16 @@ class Cgroup(object):
 
     @staticmethod
     def get_controller_mount_point(ctrl_name):
-        vers = CgroupVersion.get_version(ctrl_name)
+        if ctrl_name is None:
+            mode = int(Cgroup.get_cgroup_mode(None))
+            # map return modes:
+            # CGROUP_MODE_LEGACY/CGROUP_MODE_HYBRID ->  CgroupVersion.CGROUP_V1
+            # CGROUP_MODE_UNIFIED -> CgroupVersion.CGROUP_V2
+            if mode >= 2:
+                mode = mode - 1
+            vers = CgroupVersion(mode)
+        else:
+            vers = CgroupVersion.get_version(ctrl_name)
 
         if vers == CgroupVersion.CGROUP_V1:
             return Cgroup.__get_controller_mount_point_v1(ctrl_name)
@@ -834,6 +923,13 @@ class Cgroup(object):
                 if mount.version == CgroupVersion.CGROUP_V1 or \
                    expand_v2_mounts is False:
                     mount_list.append(mount)
+
+                    if entry[1].find(',') > 0:
+                        # multiple controllers are mounted together.  Also add the
+                        # first controller to the mount_list
+                        controller = os.path.basename(entry[1].split(',')[0])
+                        mount = CgroupMount(line, controller=controller)
+                        mount_list.append(mount)
                     continue
 
                 with open(os.path.join(mount.mount_point,
@@ -876,7 +972,7 @@ class Cgroup(object):
     # exec is a keyword in python, so let's name this function cgexec
     @staticmethod
     def cgexec(config, controller, cgname, cmdline, sticky=False,
-               cghelp=False):
+               cghelp=False,  ignore_systemd=False, replace_idle=False):
         """cgexec equivalent method
         """
         cmd = list()
@@ -884,6 +980,13 @@ class Cgroup(object):
         if not config.args.container:
             cmd.append('sudo')
         cmd.append(Cgroup.build_cmd_path('cgexec'))
+
+        if (ignore_systemd):
+            cmd.append('-b')
+
+        if (replace_idle):
+            cmd.append('-r')
+
         cmd.append('-g')
         cmd.append('{}:{}'.format(controller, cgname))
 
@@ -894,7 +997,7 @@ class Cgroup(object):
             cmd.append(cmdline)
         elif isinstance(cmdline, list):
             for entry in cmdline:
-                cmd.append(entry)
+                cmd.append(str(entry))
 
         if cghelp:
             cmd.append('-h')
@@ -918,14 +1021,20 @@ class Cgroup(object):
                 cmd = ['cat', proc_file]
 
                 if config.args.container:
-                    return config.container.run(cmd, shell_bool=True)
+                    pids = config.container.run(cmd, shell_bool=True)
                 else:
-                    return Run.run(cmd, shell_bool=True)
+                    pids = Run.run(cmd, shell_bool=True)
+
+                pid_list = list()
+                for pid in pids.splitlines():
+                    pid_list.append(int(pid))
+
+                return pid_list
 
         return None
 
     @staticmethod
-    def get_and_validate(config, cgname, setting, expected_value):
+    def get_and_validate(config, cgname, setting, expected_value, ignore_systemd=False):
         """get the requested setting and validate the value received
 
         This is a helper method for the functional tests and there is no
@@ -934,22 +1043,153 @@ class Cgroup(object):
         """
         value = Cgroup.get(config, controller=None, cgname=cgname,
                            setting=setting, print_headers=False,
-                           values_only=True)
+                           values_only=True, ignore_systemd=ignore_systemd)
 
         if value != expected_value:
             raise CgroupError('cgget expected {} but received {}'.format(
                               expected_value, value))
 
     @staticmethod
-    def set_and_validate(config, cgname, setting, value):
+    def set_and_validate(config, cgname, setting, value, ignore_systemd=False):
         """set the requested setting and validate the write
 
         This is a helper method for the functional tests and there is no
         equivalent libcgroup command line interface.  This method will
         raise a CgroupError if the comparison fails
         """
-        Cgroup.set(config, cgname, setting, value)
-        Cgroup.get_and_validate(config, cgname, setting, value)
+        Cgroup.set(config, cgname, setting, value, ignore_systemd=ignore_systemd)
+        Cgroup.get_and_validate(config, cgname, setting, value, ignore_systemd=ignore_systemd)
+
+    @staticmethod
+    def get_cgroup_mode(config):
+        mount_list = Cgroup.get_cgroup_mounts(config, True)
+
+        legacy = False
+        unified = False
+
+        for mount in mount_list:
+            # As best I can tell, python doesn't have an easy way to access the statfs() f_type
+            # field.  Let's make our best guess at the cgroup version (legacy, hybrid, or
+            # unified) by the presence of certain files/folders
+
+            cpu_ctrl = os.path.join(os.path.dirname(mount.mount_point), 'cpu,cpuacct')
+            if os.path.exists(cpu_ctrl):
+                legacy = True
+
+            limit_file = os.path.join(mount.mount_point, 'memory.limit_in_bytes')
+            if os.path.exists(limit_file):
+                legacy = True
+
+            subtree_file = os.path.join(mount.mount_point, 'cgroup.subtree_control')
+            if os.path.exists(subtree_file):
+                unified = True
+
+            subtree_file = os.path.join(os.path.dirname(mount.mount_point),
+                                        'cgroup.subtree_control')
+            if os.path.exists(subtree_file):
+                unified = True
+
+            subtree_file = os.path.join(os.path.dirname(mount.mount_point), 'unified',
+                                        'cgroup.subtree_control')
+            if os.path.exists(subtree_file):
+                unified = True
+
+        if legacy and unified:
+            return Mode.CGROUP_MODE_HYBRID
+        elif legacy:
+            return Mode.CGROUP_MODE_LEGACY
+        elif unified:
+            return Mode.CGROUP_MODE_UNIFIED
+        else:
+            raise CgroupError('Unknown cgroup mode')
+
+    @staticmethod
+    def is_controller_enabled(config, cgroup_name, ctrl_name):
+        ctrl_path = Cgroup.__get_controller_mount_point_v2(ctrl_name)
+        parent_cgname = os.path.dirname(cgroup_name)
+
+        subtree_path = os.path.join(ctrl_path, parent_cgname, 'cgroup.controllers')
+        cmd = ['cat', subtree_path]
+
+        if config.args.container:
+            controllers = config.container.run(cmd, shell_bool=True)
+        else:
+            controllers = Run.run(cmd, shell_bool=True)
+
+        for controller in controllers.split():
+            if controller == ctrl_name:
+                return True
+
+        return False
+
+    @staticmethod
+    def __exists(config, ctrl_name, cgroup_name, ignore_systemd):
+        ctrl_mnt = Cgroup.get_controller_mount_point(ctrl_name)
+
+        if (ignore_systemd):
+            cgrp_path = os.path.join(ctrl_mnt, cgroup_name)
+        else:
+            try:
+                cmd = ['cat', '/run/libcgroup/systemd']
+                if config.args.container:
+                    slice_scope_path = config.container.run(cmd, shell_bool=True)
+                else:
+                    slice_scope_path = Run.run(cmd, shell_bool=True)
+            except RunError:
+                return False
+
+            cgrp_path = os.path.join(ctrl_mnt, slice_scope_path, cgroup_name)
+
+        # Every cgroup is populated with cgroup.procs (setup mode agnostic)
+        # It easier to check of the directory exist by tring to read it
+        proc_file = os.path.join(cgrp_path, 'cgroup.procs')
+        cmd = ['cat', proc_file]
+        try:
+            if config.args.container:
+                config.container.run(cmd, shell_bool=True)
+            else:
+                Run.run(cmd, shell_bool=True)
+        except RunError:
+            return False
+
+        return True
+
+    # This function builds absolute path of the cgroup, using the default
+    # systemd path and checks if the cgroup exists. The function can build
+    # path for all three cgroup setup modes, including empty cgroup v2.
+    @staticmethod
+    def exists(config, ctrl_name, cgroup_name, ignore_systemd=False):
+        if ctrl_name is None or type(ctrl_name) == str:
+            return Cgroup.__exists(config, ctrl_name, cgroup_name, ignore_systemd)
+        elif type(ctrl_name) == list:
+            for ctrl in ctrl_name:
+                if not Cgroup.__exists(config, ctrl, cgroup_name, ignore_systemd):
+                    # if any of the controllers don't exist, fail the check
+                    return False
+
+            # all of the controllers exist,  Yippee
+            return True
+        else:
+            raise ValueError("Unsupported type: {}".format(type(ctrl_name)))
+
+    # Method to enable or disable controllers in the subtree control file
+    @staticmethod
+    def subtree_control(config, cgname, controllers, enable=True, ignore_systemd=False):
+        if isinstance(controllers, str):
+            controllers = [controllers]
+        elif isinstance(controllers, list):
+            pass
+        else:
+            raise ValueError('Unsupported controller value')
+
+        if enable:
+            enable_char = '+'
+        else:
+            enable_char = '-'
+
+        for ctrl in controllers:
+            Cgroup.set(config, cgname, 'cgroup.subtree_control',
+                       '{}{}'.format(enable_char, ctrl), ignore_systemd=ignore_systemd)
 
 
 class CgroupError(Exception):
