@@ -220,9 +220,6 @@ def setup(config, do_teardown=True, record_time=False):
         config.container.config()
         config.container.start()
 
-        # LXC on Ubuntu 20.04 put sed in a different spot.  Add a symlink
-        config.container.run(['ln', '-s', '/bin/sed', '/usr/bin/sed'])
-
         # add the libcgroup library to the container's ld
         libcgrp_lib_path = os.path.join(consts.LIBCG_MOUNT_POINT, 'src/.libs')
         echo_cmd = ([
@@ -277,6 +274,13 @@ def run_tests(config):
                 if config.args.num == consts.TESTS_RUN_ALL or \
                    config.args.num == filenum_int:
 
+                    if config.args.suite == consts.TESTS_RUN_ALL_SUITES and \
+                       filesuite == 'sudo':
+                        # Don't run the 'sudo' tests if all tests have been specified.
+                        # The sudo tests must be run as sudo and thus need to be separately
+                        # invoked.
+                        continue
+
                     if filenum_int in config.skip_list:
                         continue
 
